@@ -6,62 +6,57 @@
 
 #![cfg_attr(doc_cfg, feature(doc_cfg))]
 
-use std::{fmt::Debug, hash::Hash, path::PathBuf};
-
-use serde::{Deserialize, Serialize};
-use tauri_utils::assets::Assets;
+use serde::Deserialize;
+use std::{fmt::Debug, path::PathBuf, sync::mpsc::Sender};
 use uuid::Uuid;
+
 #[cfg(windows)]
 use winapi::shared::windef::HWND;
 
 /// Create window and system tray menus.
-#[cfg(any(feature = "menu", feature = "system-tray"))]
-#[cfg_attr(doc_cfg, doc(cfg(any(feature = "menu", feature = "system-tray"))))]
 pub mod menu;
 /// Types useful for interacting with a user's monitors.
 pub mod monitor;
-pub mod tag;
 pub mod webview;
 pub mod window;
 
 use monitor::Monitor;
-use tag::Tag;
 use webview::WindowBuilder;
 use window::{
   dpi::{PhysicalPosition, PhysicalSize, Position, Size},
   DetachedWindow, PendingWindow, WindowEvent,
 };
 
-/// A type that can be derived into a menu id.
-pub trait MenuId: Serialize + Hash + Eq + Debug + Clone + Send + Sync + 'static {}
-
-impl<T> MenuId for T where T: Serialize + Hash + Eq + Debug + Clone + Send + Sync + 'static {}
-
 #[cfg(feature = "system-tray")]
 #[non_exhaustive]
-pub struct SystemTray<I: MenuId> {
+#[derive(Debug)]
+pub struct SystemTray {
   pub icon: Option<Icon>,
-  pub menu: Option<menu::SystemTrayMenu<I>>,
+  pub menu: Option<menu::SystemTrayMenu>,
+  #[cfg(target_os = "macos")]
+  pub icon_as_template: bool,
 }
 
 #[cfg(feature = "system-tray")]
-impl<I: MenuId> Default for SystemTray<I> {
+impl Default for SystemTray {
   fn default() -> Self {
     Self {
       icon: None,
       menu: None,
+      #[cfg(target_os = "macos")]
+      icon_as_template: false,
     }
   }
 }
 
 #[cfg(feature = "system-tray")]
-impl<I: MenuId> SystemTray<I> {
+impl SystemTray {
   /// Creates a new system tray that only renders an icon.
   pub fn new() -> Self {
     Default::default()
   }
 
-  pub fn menu(&self) -> Option<&menu::SystemTrayMenu<I>> {
+  pub fn menu(&self) -> Option<&menu::SystemTrayMenu> {
     self.menu.as_ref()
   }
 
@@ -71,8 +66,15 @@ impl<I: MenuId> SystemTray<I> {
     self
   }
 
+  /// Sets the tray icon as template.
+  #[cfg(target_os = "macos")]
+  pub fn with_icon_as_template(mut self, is_template: bool) -> Self {
+    self.icon_as_template = is_template;
+    self
+  }
+
   /// Sets the menu to show when the system tray is right clicked.
-  pub fn with_menu(mut self, menu: menu::SystemTrayMenu<I>) -> Self {
+  pub fn with_menu(mut self, menu: menu::SystemTrayMenu) -> Self {
     self.menu.replace(menu);
     self
   }
@@ -126,32 +128,6 @@ pub enum Error {
 /// Result type.
 pub type Result<T> = std::result::Result<T, Error>;
 
-#[doc(hidden)]
-pub mod private {
-  pub trait ParamsBase {}
-}
-
-/// Types associated with the running Tauri application.
-pub trait Params: private::ParamsBase + 'static {
-  /// The event type used to create and listen to events.
-  type Event: Tag;
-
-  /// The type used to determine the name of windows.
-  type Label: Tag;
-
-  /// The type used to determine window menu ids.
-  type MenuId: MenuId;
-
-  /// The type used to determine system tray menu ids.
-  type SystemTrayMenuId: MenuId;
-
-  /// Assets that Tauri should serve from itself.
-  type Assets: Assets;
-
-  /// The underlying webview runtime used by the Tauri application.
-  type Runtime: Runtime;
-}
-
 /// A icon definition.
 #[derive(Debug, Clone)]
 #[non_exhaustive]
@@ -189,14 +165,44 @@ impl Icon {
 }
 
 /// Event triggered on the event loop run.
+#[non_exhaustive]
 pub enum RunEvent {
   /// Event loop is exiting.
   Exit,
+  /// Event loop is about to exit
+  ExitRequested {
+    /// Label of the last window managed by the runtime.
+    window_label: String,
+    tx: Sender<ExitRequestedEventAction>,
+  },
+  /// Window close was requested by the user.
+  CloseRequested {
+    /// The window label.
+    label: String,
+    /// A signal sender. If a `true` value is emitted, the window won't be closed.
+    signal_tx: Sender<bool>,
+  },
   /// Window closed.
   WindowClose(String),
+  /// Application ready.
+  Ready,
+  /// Sent if the event loop is being resumed.
+  Resumed,
+  /// Emitted when all of the event loop’s input events have been processed and redraw processing is about to begin.
+  ///
+  /// This event is useful as a place to put your code that should be run after all state-changing events have been handled and you want to do stuff (updating state, performing calculations, etc) that happens as the “main body” of your event loop.
+  MainEventsCleared,
+}
+
+/// Action to take when the event loop is about to exit
+#[derive(Debug)]
+pub enum ExitRequestedEventAction {
+  /// Prevent the event loop from exiting
+  Prevent,
 }
 
 /// A system tray event.
+#[derive(Debug)]
 pub enum SystemTrayEvent {
   MenuItemClick(u16),
   LeftClick {
@@ -216,17 +222,30 @@ pub enum SystemTrayEvent {
 /// Metadata for a runtime event loop iteration on `run_iteration`.
 #[derive(Debug, Clone, Default)]
 pub struct RunIteration {
-  pub webview_count: usize,
+  pub window_count: usize,
+}
+
+/// Application's activation policy. Corresponds to NSApplicationActivationPolicy.
+#[cfg(target_os = "macos")]
+#[cfg_attr(doc_cfg, doc(cfg(target_os = "macos")))]
+#[non_exhaustive]
+pub enum ActivationPolicy {
+  /// Corresponds to NSApplicationActivationPolicyRegular.
+  Regular,
+  /// Corresponds to NSApplicationActivationPolicyAccessory.
+  Accessory,
+  /// Corresponds to NSApplicationActivationPolicyProhibited.
+  Prohibited,
 }
 
 /// A [`Send`] handle to the runtime.
-pub trait RuntimeHandle: Send + Sized + Clone + 'static {
+pub trait RuntimeHandle: Debug + Send + Sized + Clone + 'static {
   type Runtime: Runtime<Handle = Self>;
   /// Create a new webview window.
-  fn create_window<P: Params<Runtime = Self::Runtime>>(
+  fn create_window(
     &self,
-    pending: PendingWindow<P>,
-  ) -> crate::Result<DetachedWindow<P>>;
+    pending: PendingWindow<Self::Runtime>,
+  ) -> crate::Result<DetachedWindow<Self::Runtime>>;
 
   #[cfg(all(windows, feature = "system-tray"))]
   #[cfg_attr(doc_cfg, doc(cfg(all(windows, feature = "system-tray"))))]
@@ -234,21 +253,25 @@ pub trait RuntimeHandle: Send + Sized + Clone + 'static {
 }
 
 /// A global shortcut manager.
-pub trait GlobalShortcutManager {
+pub trait GlobalShortcutManager: Debug {
   /// Whether the application has registered the given `accelerator`.
   ///
   /// # Panics
   ///
-  /// Panics if the app is not running yet, usually when called on the `tauri::Builder#setup` closure.
-  /// You can spawn a task to use the API using the `tauri::async_runtime` to prevent the panic.
+  /// - Panics if the event loop is not running yet, usually when called on the `tauri::Builder#setup` closure.
+  /// - Panics when called on the main thread, usually on the `tauri::App#run`closure.
+  ///
+  /// You can spawn a task to use the API using `tauri::async_runtime::spawn` or [`std::thread::spawn`] to prevent the panic.
   fn is_registered(&self, accelerator: &str) -> crate::Result<bool>;
 
   /// Register a global shortcut of `accelerator`.
   ///
   /// # Panics
   ///
-  /// Panics if the app is not running yet, usually when called on the `tauri::Builder#setup` closure.
-  /// You can spawn a task to use the API using the `tauri::async_runtime` to prevent the panic.
+  /// - Panics if the event loop is not running yet, usually when called on the `tauri::Builder#setup` closure.
+  /// - Panics when called on the main thread, usually on the `tauri::App#run`closure.
+  ///
+  /// You can spawn a task to use the API using `tauri::async_runtime::spawn` or [`std::thread::spawn`] to prevent the panic.
   fn register<F: Fn() + Send + 'static>(
     &mut self,
     accelerator: &str,
@@ -259,35 +282,42 @@ pub trait GlobalShortcutManager {
   ///
   /// # Panics
   ///
-  /// Panics if the app is not running yet, usually when called on the `tauri::Builder#setup` closure.
-  /// You can spawn a task to use the API using the `tauri::async_runtime` to prevent the panic.
+  /// - Panics if the event loop is not running yet, usually when called on the `tauri::Builder#setup` closure.
+  /// - Panics when called on the main thread, usually on the `tauri::App#run`closure.
+  ///
+  /// You can spawn a task to use the API using `tauri::async_runtime::spawn` or [`std::thread::spawn`] to prevent the panic.
   fn unregister_all(&mut self) -> crate::Result<()>;
 
   /// Unregister the provided `accelerator`.
   ///
   /// # Panics
   ///
-  /// Panics if the app is not running yet, usually when called on the `tauri::Builder#setup` closure.
-  /// You can spawn a task to use the API using the `tauri::async_runtime` to prevent the panic.
+  /// - Panics if the event loop is not running yet, usually when called on the `tauri::Builder#setup` closure.
+  /// - Panics when called on the main thread, usually on the `tauri::App#run`closure.
+  ///
+  /// You can spawn a task to use the API using `tauri::async_runtime::spawn` or [`std::thread::spawn`] to prevent the panic.
   fn unregister(&mut self, accelerator: &str) -> crate::Result<()>;
 }
 
 /// Clipboard manager.
-pub trait ClipboardManager {
+pub trait ClipboardManager: Debug {
   /// Writes the text into the clipboard as plain text.
   ///
   /// # Panics
   ///
-  /// Panics if the app is not running yet, usually when called on the `tauri::Builder#setup` closure.
-  /// You can spawn a task to use the API using the `tauri::async_runtime` to prevent the panic.
-
+  /// - Panics if the event loop is not running yet, usually when called on the `tauri::Builder#setup` closure.
+  /// - Panics when called on the main thread, usually on the `tauri::App#run`closure.
+  ///
+  /// You can spawn a task to use the API using `tauri::async_runtime::spawn` or [`std::thread::spawn`] to prevent the panic.
   fn write_text<T: Into<String>>(&mut self, text: T) -> Result<()>;
   /// Read the content in the clipboard as plain text.
   ///
   /// # Panics
   ///
-  /// Panics if the app is not running yet, usually when called on the `tauri::Builder#setup` closure.
-  /// You can spawn a task to use the API using the `tauri::async_runtime` to prevent the panic.
+  /// - Panics if the event loop is not running yet, usually when called on the `tauri::Builder#setup` closure.
+  /// - Panics when called on the main thread, usually on the `tauri::App#run`closure.
+  ///
+  /// You can spawn a task to use the API using `tauri::async_runtime::spawn` or [`std::thread::spawn`] to prevent the panic.
   fn read_text(&self) -> Result<Option<String>>;
 }
 
@@ -318,21 +348,23 @@ pub trait Runtime: Sized + 'static {
   fn clipboard_manager(&self) -> Self::ClipboardManager;
 
   /// Create a new webview window.
-  fn create_window<P: Params<Runtime = Self>>(
-    &self,
-    pending: PendingWindow<P>,
-  ) -> crate::Result<DetachedWindow<P>>;
+  fn create_window(&self, pending: PendingWindow<Self>) -> crate::Result<DetachedWindow<Self>>;
 
   /// Adds the icon to the system tray with the specified menu items.
   #[cfg(feature = "system-tray")]
   #[cfg_attr(doc_cfg, doc(cfg(feature = "system-tray")))]
-  fn system_tray<I: MenuId>(&self, system_tray: SystemTray<I>) -> crate::Result<Self::TrayHandler>;
+  fn system_tray(&self, system_tray: SystemTray) -> crate::Result<Self::TrayHandler>;
 
   /// Registers a system tray event handler.
   #[cfg(feature = "system-tray")]
   #[cfg_attr(doc_cfg, doc(cfg(feature = "system-tray")))]
   fn on_system_tray_event<F: Fn(&SystemTrayEvent) + Send + 'static>(&mut self, f: F) -> Uuid;
 
+  /// Sets the activation policy for the application. It is set to `NSApplicationActivationPolicyRegular` by default.
+  #[cfg(target_os = "macos")]
+  #[cfg_attr(doc_cfg, doc(cfg(target_os = "macos")))]
+  fn set_activation_policy(&mut self, activation_policy: ActivationPolicy);
+
   /// Runs the one step of the webview runtime event loop and returns control flow to the caller.
   #[cfg(any(target_os = "windows", target_os = "macos"))]
   fn run_iteration<F: Fn(RunEvent) + 'static>(&mut self, callback: F) -> RunIteration;
@@ -342,7 +374,7 @@ pub trait Runtime: Sized + 'static {
 }
 
 /// Webview dispatcher. A thread-safe handle to the webview API.
-pub trait Dispatch: Clone + Send + Sized + 'static {
+pub trait Dispatch: Debug + Clone + Send + Sized + 'static {
   /// The runtime this [`Dispatch`] runs under.
   type Runtime: Runtime;
 
@@ -356,8 +388,6 @@ pub trait Dispatch: Clone + Send + Sized + 'static {
   fn on_window_event<F: Fn(&WindowEvent) + Send + 'static>(&self, f: F) -> Uuid;
 
   /// Registers a window event handler.
-  #[cfg(feature = "menu")]
-  #[cfg_attr(doc_cfg, doc(cfg(feature = "menu")))]
   fn on_menu_event<F: Fn(&window::MenuEvent) + Send + 'static>(&self, f: F) -> Uuid;
 
   // GETTERS
@@ -397,7 +427,6 @@ pub trait Dispatch: Clone + Send + Sized + 'static {
   fn is_visible(&self) -> crate::Result<bool>;
 
   /// Gets the window menu current visibility state.
-  #[cfg(feature = "menu")]
   fn is_menu_visible(&self) -> crate::Result<bool>;
 
   /// Returns the monitor on which the window currently resides.
@@ -417,6 +446,20 @@ pub trait Dispatch: Clone + Send + Sized + 'static {
   #[cfg(windows)]
   fn hwnd(&self) -> crate::Result<HWND>;
 
+  /// Returns the native handle that is used by this window.
+  #[cfg(target_os = "macos")]
+  fn ns_window(&self) -> crate::Result<*mut std::ffi::c_void>;
+
+  /// Returns the `ApplicatonWindow` from gtk crate that is used by this window.
+  #[cfg(any(
+    target_os = "linux",
+    target_os = "dragonfly",
+    target_os = "freebsd",
+    target_os = "netbsd",
+    target_os = "openbsd"
+  ))]
+  fn gtk_window(&self) -> crate::Result<gtk::ApplicationWindow>;
+
   // SETTERS
 
   /// Centers the window.
@@ -431,10 +474,10 @@ pub trait Dispatch: Clone + Send + Sized + 'static {
   fn request_user_attention(&self, request_type: Option<UserAttentionType>) -> crate::Result<()>;
 
   /// Create a new webview window.
-  fn create_window<P: Params<Runtime = Self::Runtime>>(
+  fn create_window(
     &mut self,
-    pending: PendingWindow<P>,
-  ) -> crate::Result<DetachedWindow<P>>;
+    pending: PendingWindow<Self::Runtime>,
+  ) -> crate::Result<DetachedWindow<Self::Runtime>>;
 
   /// Updates the window resizable flag.
   fn set_resizable(&self, resizable: bool) -> crate::Result<()>;
@@ -455,11 +498,9 @@ pub trait Dispatch: Clone + Send + Sized + 'static {
   fn unminimize(&self) -> crate::Result<()>;
 
   /// Shows the window menu.
-  #[cfg(feature = "menu")]
   fn show_menu(&self) -> crate::Result<()>;
 
   /// Hides the window menu.
-  #[cfg(feature = "menu")]
   fn hide_menu(&self) -> crate::Result<()>;
 
   /// Shows the window.
@@ -508,6 +549,5 @@ pub trait Dispatch: Clone + Send + Sized + 'static {
   fn eval_script<S: Into<String>>(&self, script: S) -> crate::Result<()>;
 
   /// Applies the specified `update` to the menu item associated with the given `id`.
-  #[cfg(feature = "menu")]
   fn update_menu_item(&self, id: u16, update: menu::MenuUpdate) -> crate::Result<()>;
 }
